#!/usr/bin/perl -w
##############################################################################
# Prepare data about the given application to be inserted into the database.
#
# This script takes a list of components to be processed from file and
# calls 'componenttodb.pl' for each file. The results are collected in
# one file that can be used by 'upload_app_data.pl' for adding information
# to the database.
#
# Copyright (C) 2007-2010 The Linux Foundation
# Copyright (C) 2007-2010 Institute for System Programming, RAS
#
# Author: Denis Silakov <silakov@ispras.ru>
##############################################################################

use strict;
use Getopt::Long;
use Text::Wrap;
use File::Temp qw/ tempfile tempdir /;
Getopt::Long::Configure ("posix_default", "no_ignore_case");

use Distros;
# Application information
my $AppName="";
my $AppVer="";
my $ComponentsFile;
my ($RpmFile, $RpmFileList, $DebFile, $DebFileList);
my $AppFilesList;
my $Comment='';
my $Summary='';
my $Aarch="";
my $OutputFile;
my $Packager="";
my $URL="";
my $LicenseType="";
my $AppUI="";
my $AppFuncCategory="";

# Package manager used in the distribution
my $PackageManager;

sub HELP_MESSAGE();
sub guess_platform($ );
sub guess_rpm_appname();
sub guess_deb_appname();
sub get_app_version();
sub get_app_packager();
sub get_app_license();
sub get_app_category();
sub guess_deb_arch();
sub guess_rpm_arch();
sub guess_category($ );

# options
my $opt_help;
my $opt_java=0;
my $opt_pkginfo=0;
my $quiet=1;
my $totally_quiet;
my $trsp_symlink;

GetOptions("help|h!" => \$opt_help,
            "comment|c=s" => \$Comment,
            "summary|s=s" => \$Summary,
            "category=s" => \$AppFuncCategory,
            "outfile|o=s" => \$OutputFile,
            "arch|a=s" => \$Aarch,
            "java" => sub {$opt_java=1;},
            "pkginfo!" => \$opt_pkginfo,
            "version|v=s" => \$AppVer,
            "licensetype|t=s" => \$LicenseType,
            "ui=s" => \$AppUI,
            "infile|i=s" => \$ComponentsFile,
            "rpm|r=s" => \$RpmFile,
            "rpmlist=s" => \$RpmFileList,
            "deb|d=s" => \$DebFile,
            "deblist=s" => \$DebFileList,
            "filelist|l=s" => \$AppFilesList,
            "packagemanager|m=s" => \$PackageManager,
            "packager|p=s" => \$Packager,
            "url|u=s" => \$URL,
            "verbose" => sub {$quiet=0;},
            "quiet|q!" => \$totally_quiet,
            "trsp-symlink" => sub {$trsp_symlink=1;},
            );

HELP_MESSAGE() if (defined $opt_help);
HELP_MESSAGE() if(@ARGV != 1 and !$DebFile and !$RpmFile);

# Flags indicating if we are processing tarball
my ($tgz,$tbz2,$txz,$zip) = (0,0,0,0);
my $tarball_name_esc = "";

# Name should be provided as argument, but for single rpms and debs we can extract it from file
if( @ARGV == 1 ) {
    # Allow rpm/deb/tgz/tbz2/zip file name to be an argument
    if( -e ($ARGV[0]) ) {
        if( $ARGV[0] =~ /\.rpm$/i ) {
            $RpmFile = $ARGV[0];
            $AppName = guess_rpm_appname();
            if( !$AppName ) {
                $RpmFile = "";
            }
        }
        elsif( $ARGV[0] =~ /\.deb$/i ) {
            $DebFile = $ARGV[0];
            $AppName = guess_deb_appname();
            if( !$AppName ) {
                $DebFile = "";
            }
        }
        elsif( $ARGV[0] =~ /^(.+)\.tgz/i or $ARGV[0] =~ /^(.+)\.tar\.gz$/i ) {
            $tarball_name_esc = escape_shell_symbols($ARGV[0]);
            $AppName = $1;
            $tgz = 1;
        }
        elsif( $ARGV[0] =~ /^(.+)\.zip/i ) {
            $tarball_name_esc = escape_shell_symbols($ARGV[0]);
            $AppName = $1;
            $zip = 1;
        }
        elsif( $ARGV[0] =~ /^(.+)\.tbz$/i or $ARGV[0] =~ /^(.+)\.tbz2$/i or $ARGV[0] =~ /^(.+)\.tb2$/i or $ARGV[0] =~ /^(.+)\.tar\.bz2$/i ) {
            $tarball_name_esc = escape_shell_symbols($ARGV[0]);
            $AppName = $1;
            $tbz2 = 1;
        }
        elsif( $ARGV[0] =~ /^(.+)\.txz/i or $ARGV[0] =~ /^(.+)\.tar\.xz$/i or $ARGV[0] =~ /^(.+)\.tlz/i or $ARGV[0] =~ /^(.+)\.tar\.lzma$/i ) {
            $tarball_name_esc = escape_shell_symbols($ARGV[0]);
            $AppName = $1;
            $txz = 1;
        }
    }

    if( !$AppName ) {
        $AppName = $ARGV[0];
    }
}
elsif( $DebFile ) {
    $AppName = guess_deb_appname();
}
elsif( $RpmFile ) {
    $AppName = guess_rpm_appname();
}

# Guess package manager, if it was not specified
if( !$AppFilesList and !$PackageManager and !$RpmFile and !$RpmFileList and !$DebFile and !$DebFileList ) {
    $PackageManager = guess_package_manager();
    if( $PackageManager eq "unknown" ) {
        die "Cannot guess package manager! Please specify it directly using '-m' option.";
    }
    else {
        warn "No package manager was specified, guessing it is $PackageManager";
    }
}

if( !$AppName ) {
    die "Cannot guess application name. Please specify it manually.";
}

# Check for required tools
if( ($RpmFile or $RpmFileList) and !does_shell_know("rpm") ) {
    die "You are trying to process rpm file(s), but you don't have rpm installed in your system.";
}

if( ($RpmFile or $RpmFileList) and !does_shell_know("rpm2cpio") ) {
    die "You are trying to process rpm file(s), but you don't have rpm2cpio installed in your system.";
}

if( ($DebFile or $DebFileList) and !does_shell_know("dpkg-deb") ) {
    die "You are trying to process deb file(s), but you don't have dpkg-deb installed in your system.";
}

# Try to guess some application properties if they are not provided through options
if( !$Packager and !$RpmFileList and !$DebFileList ) {
    if( $AppFilesList or $ComponentsFile ) {
        print STDERR "You should specify packager manually when you use files or components list.\n";
    }
    else {
        $Packager = get_app_packager();
    }
}

# Auxilliary vars for get_component_* routines
my $mode = "";
my $AnalyzeName = $AppName;
if( $RpmFile ) {
    $mode = "rpm";
    $AnalyzeName = $RpmFile;
}
elsif( $DebFile ) {
    $mode = "deb";
    $AnalyzeName = $DebFile;
}
elsif( $RpmFileList ) {
    # For list of rpms, we'll try to guess summary/url using the first file
    $mode = "rpm";
    my $RpmFileList_esc = escape_shell_symbols($RpmFileList);
    $AnalyzeName = `head -1 $RpmFileList_esc`;
    chomp($AnalyzeName);
}
elsif( $DebFileList ) {
    # For list of debs, we'll try to guess summary/url using the first file
    $mode = "deb";
    my $DebFileList_esc = escape_shell_symbols($DebFileList);
    $AnalyzeName = `head -1 $DebFileList_esc`;
    chomp($AnalyzeName);
}

if( !$URL ) {
    if( $AppFilesList or $ComponentsFile ) {
        print STDERR "You should specify URL manually when you use files or components list.\n";
    }
    else {
        $URL = get_component_url("application", $AnalyzeName, $mode, $PackageManager, $totally_quiet);
    }
}

if( !$LicenseType ) {
    if( $AppFilesList or $ComponentsFile or $RpmFileList or $DebFileList ) {
        print STDERR "You should specify license type manually when you use files or components list.\n";
    }
    else {
        $LicenseType = get_app_license();
    }
}

if( !$AppFuncCategory ) {
    if( $AppFilesList or $ComponentsFile or $RpmFileList or $DebFileList ) {
        print STDERR "You should specify application functional category manually when you use files or components list.\n";
    }
    else {
        $AppFuncCategory = get_app_category();
    }
}

if( !$AppVer ) {
    if( $AppFilesList or $ComponentsFile or $RpmFileList or $DebFileList ) {
        print STDERR "You should specify version manually when you use files or components list.\n";
    }
    else {
        $AppVer = get_app_version();
    }
}

# Do not allow illegal UI values
if( $AppUI and ($AppUI ne "GUI") and ($AppUI ne "non-GUI") ) {
    $AppUI = "";
}

# Normally, we use 'x86', but db only knows about 'IA32'
if( $Aarch eq 'x86' ) {
    $Aarch = 'IA32';
}

# Guess application architecture, if it was not specified
if( not $Aarch ) {
    if( $DebFile ) {
        $Aarch = guess_deb_arch();
    }
    elsif( $RpmFile ) {
        $Aarch = guess_rpm_arch();
    }

    # In we are not processing rpm or deb, just use architecture of our OS
    if( not $Aarch ) {
        $Aarch = guess_platform(`uname -m`);
    }
}

if( !$Summary ) {
    # Application summary
    if( $AppFilesList ) {
        print STDERR "If you want to add summary for you application, specify it with '-s' option.\n";
    }
    else {
        $Summary = get_component_summary("application", $AnalyzeName, $mode, $PackageManager, $totally_quiet);
    }
}
else {
    $Summary =~ s/\'/\'\^/g;
}

# default output file name
if( not $OutputFile ) {
    my $outArch = lc($Aarch);
    if( $outArch =~ /ia32/ ) {
        $outArch = 'x86';
    }
    $OutputFile = $AppName."_".$AppVer."_".$outArch."_upload_data";
    $OutputFile =~ s/\//\_/g;
}

if( -e $OutputFile ) {
    print STDERR "!!! Can't dump application data - file '$OutputFile' already exists\n";
    print STDERR "!!! Please remove or rename the file first, or specify another output file using '-o' option.\n";
    exit(1);
}

$OutputFile = escape_shell_symbols($OutputFile);

##############################################################################
# Main section
##############################################################################

system "echo \"!Application '$AppName' '$AppVer' '$Aarch' '$Packager' '$URL' '$Summary'\" > $OutputFile";
system "echo \"!AppCategory '$LicenseType' '$AppUI' '$AppFuncCategory'\" >> $OutputFile";

if( $Comment ) {
    # Do not enclose comment with quotes - let's treat the whole string after '!AppComment' as comment text
    system "echo \"!AppComment $Comment\" >> $OutputFile";
}

my $tmpDir = tempdir( CLEANUP => 1 );
$tmpDir = escape_shell_symbols($tmpDir);

# Options for componenttodb.pl
my $options = "";
if( $totally_quiet ) {
    $options.= " -q ";
}
if( !$quiet ) {
    $options.= " --verbose ";
}
if( $opt_java ) {
    $options.= " --java ";
}

# For deb and rpms - unpack contents to $tmpDir and set '-l' option for componenttodb.pl
if( $RpmFile ) {
    my $filename = escape_shell_symbols($RpmFile);
    system "rm -rf $tmpDir/*; cp $filename $tmpDir";
    my $cpio = $quiet ? "cpio -id --quiet" : "cpio -id";
    system "cd $tmpDir && rpm2cpio $filename | $cpio ";
    system "rm $tmpDir/$filename";
    system "echo $tmpDir/ > $tmpDir/apptodb_list";
    $options.= " -l $tmpDir/apptodb_list";

    if( $opt_pkginfo ) {
        system "echo \"!PkgRequires\" >> $OutputFile";
        system "rpm -qpR --nosignature $filename |LC_ALL=C sort -u >> $OutputFile";
    }
}
elsif( $RpmFileList ) {
    system "rm -rf $tmpDir; mkdir $tmpDir";
    my $cpio = $quiet ? "cpio -id --quiet" : "cpio -id";

    if( $opt_pkginfo ) {
        system "echo \"!PkgRequires\" >> $OutputFile";
    }

    open( RPM_LIST, "<$RpmFileList" );
    while( <RPM_LIST> )
    {
        chomp;
        if( $_ ) {
            if( !-e $_ ) {
                warn "File '$_' from the list doesn't exist!" if(!$totally_quiet);
                next;
            }
            my $filename = escape_shell_symbols($_);
            my $base=`basename $filename`;
            chomp($base);
            $base = escape_shell_symbols($base);
            system "cp $filename $tmpDir/ ";
            system "cd $tmpDir/ && rpm2cpio $base | $cpio ";
            system "rm $tmpDir/$base";

            if( $opt_pkginfo ) {
                system "rpm -qpR --nosignature $filename |LC_ALL=C sort -u >> $OutputFile";
            }
        }
    }
    close( RPM_LIST );

    system "echo $tmpDir/ > $tmpDir/apptodb_list";
    $options.= " -l $tmpDir/apptodb_list";
}
elsif( $DebFile ) {
    my $filename = escape_shell_symbols($DebFile);
    system "rm -rf $tmpDir/*; cp $filename $tmpDir";
    system "cd $tmpDir && ar -x $filename";
    system "rm -f $tmpDir/control.tar.gz $tmpDir/debian-binary $tmpDir/$DebFile";
    if( -e "$tmpDir/data.tar.gz" ) {
        system( "cd $tmpDir && tar xizf data.tar.gz && rm -f data.tar.gz" ) == 0 or die();
    }
    elsif( -e "$tmpDir/data.tar.bz2" ) {
        system( "cd $tmpDir && tar xijf data.tar.bz2 && rm -f data.tar.bz2" ) == 0 or die();
    }
    elsif( -e "$tmpDir/data.tar.lzma" ) {
        system( "cd $tmpDir && lzma -d data.tar.lzma && tar xif data.tar && rm -f data.tar" ) == 0 or die();
    }
    else {
        die "Neither data.tar.gz nor data.tar.lzma nor data.tar.bz2 files were found in the deb package";
    }

    system "echo $tmpDir/ > $tmpDir/apptodb_list";
    $options.= " -l $tmpDir/apptodb_list";
}
elsif( $DebFileList ) {
    system "rm -rf $tmpDir/*";

    open( RPM_LIST, "<$DebFileList" );
    while( <RPM_LIST> )
    {
        chomp;
        if( $_ ) {
            if( !-e $_ ) {
                warn "File '$_' from the list doesn't exist!" if(!$totally_quiet);
                next;
            }
            my $filename = escape_shell_symbols($_);
            my $base=`basename $filename`;
            chomp($base);
            $base = escape_shell_symbols($base);
            system "cp $filename $tmpDir";
            system "cd $tmpDir && ar -x $base";
            system "rm -f $tmpDir/control.tar.gz $tmpDir/debian-binary $tmpDir/$base";
            if( -e "$tmpDir/data.tar.gz" ) {
                system( "cd $tmpDir && tar xzf data.tar.gz && rm -f data.tar.gz" ) == 0 or die();
            }
            elsif( -e "$tmpDir/data.tar.bz2" ) {
                system( "cd $tmpDir && tar xjf data.tar.bz2 && rm -f data.tar.bz2" ) == 0 or die();
            }
            elsif( -e "$tmpDir/data.tar.lzma" ) {
                system( "cd $tmpDir && lzma -d data.tar.lzma && tar xf data.tar && rm -f data.tar" ) == 0 or die();
            }
            else {
                die "Neither data.tar.gz nor data.tar.lzma nor data.tar.bz2 files were found in the deb package $base";
            }
        }
    }
    close( RPM_LIST );

    system "echo $tmpDir/ > $tmpDir/apptodb_list";
    $options.= " -l $tmpDir/apptodb_list";
}
elsif( $AppFilesList ) {
    $AppFilesList = escape_shell_symbols($AppFilesList);
    $options.= " -l $AppFilesList ";
}
elsif( $tgz ) {
    my $tarball_base = `basename $tarball_name_esc`;
    chomp($tarball_base);
    my $tarball_base_esc = escape_shell_symbols($tarball_base);
    system "cp $tarball_name_esc $tmpDir && cd $tmpDir && tar xizf $tarball_base_esc && rm -f $tarball_base_esc";
    system "echo $tmpDir/ > $tmpDir/apptodb_list";
    $options.= " -l $tmpDir/apptodb_list";
}
elsif( $zip ) {
    my $tarball_base = `basename $tarball_name_esc`;
    chomp($tarball_base);
    my $tarball_base_esc = escape_shell_symbols($tarball_base);
    system "cp $tarball_name_esc $tmpDir && cd $tmpDir && unzip -qq $tarball_base_esc && rm -f $tarball_base_esc";
    system "echo $tmpDir/ > $tmpDir/apptodb_list";
    $options.= " -l $tmpDir/apptodb_list";
}
elsif( $tbz2 ) {
    my $tarball_base = `basename $tarball_name_esc`;
    chomp($tarball_base);
    my $tarball_base_esc = escape_shell_symbols($tarball_base);
    system "cp $tarball_name_esc $tmpDir && cd $tmpDir && tar xijf $tarball_base_esc && rm -f $tarball_base_esc";
    system "echo $tmpDir/ > $tmpDir/apptodb_list";
    $options.= " -l $tmpDir/apptodb_list";
}
elsif( $txz ) {
    my $tarball_base = `basename $tarball_name_esc`;
    chomp($tarball_base);
    my $tarball_base_esc = escape_shell_symbols($tarball_base);
    my $tarball_esc = $tarball_base_esc;
    $tarball_esc =~ s/\.lzma$//;
    $tarball_esc =~ s/\.xz$//;
    $tarball_esc =~ s/tlz$/tar/;
    $tarball_esc =~ s/txz$/tar/;
    system "cp $tarball_name_esc $tmpDir && cd $tmpDir && xz -d $tarball_base_esc && tar xif $tarball_esc && rm -f $tarball_base_esc $tarball_esc";
    system "echo $tmpDir/ > $tmpDir/apptodb_list";
    $options.= " -l $tmpDir/apptodb_list";
}

if( $PackageManager ) {
    $options.= " -m $PackageManager ";
}

if( $trsp_symlink ) {
    $options.= " -trsp-symlink";
}

my $tmpfile = "/tmp/componenttodb_".$$."_upload_data";
unlink($tmpfile);

if( $ComponentsFile ) {
    open( COMPONENT_LIST, "< $ComponentsFile" );
    while( <COMPONENT_LIST> )
    {
        chomp;
        # in the input file we can find either the component name
        # and the package that provide it or only the component name
        if( /([\w:\-\.\d]+) (.+)/) {
            my $cmp_name = escape_shell_symbols($1);
            my $pkg_name = escape_shell_symbols($2);
            my $cmd = "./componenttodb.pl $options -c $cmp_name -f $tmpfile $pkg_name";
            system( $cmd ) == 0 or die "componenttodb.pl failed: '$cmd'";

            system "cat $tmpfile >> $OutputFile";
            system "rm $tmpfile";
        }
        elsif( $_ ) { # Do not process empty lines
            my $cmp_name = escape_shell_symbols($_);
            my $cmd = "./componenttodb.pl $options -f $tmpfile $cmp_name";
            system( $cmd ) == 0 or die "componenttodb.pl failed: '$cmd'";

            system "cat $tmpfile >> $OutputFile";
            system "rm $tmpfile";
        }
    }
}
else {
    # No file with packages
    my $AppName_esc = escape_shell_symbols($AppName);
    my $cmd = "./componenttodb.pl $options -f $tmpfile $AppName_esc";
    system( $cmd ) == 0 or die "componenttodb.pl failed: '$cmd'";

    system "cat $tmpfile >> $OutputFile";
    system "rm $tmpfile";

    # Delete "$tmpDir" from result file if we are processing rpms or debs
    if( $RpmFile or $RpmFileList or $DebFile or $DebFileList or $tgz or $tbz2 or $txz or $zip ) {
        my $tmpDir_escaped = $tmpDir;
        $tmpDir_escaped =~ s/\//\\\\\//g;
        system "LC_ALL=C sed -i s/$tmpDir_escaped//g $OutputFile";
    }
}

##############################################################################
# Miscellaneous functions - obtain different application information
##############################################################################

# Query package manager for application packager
# Return: Packager value
sub get_app_packager()
{
    my $PkgLine;
    my $AppName_esc = escape_shell_symbols($AppName);
    my $packager = "";

    if( $DebFile ) {
        my $filename = escape_shell_symbols($DebFile);
        $PkgLine = `dpkg-deb -I $filename | LC_ALL=C grep -F Maintainer | cut -d\\: -f2- | cut -d\\< -f1`;
        $packager = trim($PkgLine);
        if( !$packager ) {
            warn "Can't detect application packager!"  if( !$totally_quiet );
        }

        return $packager;
    }

    if( $RpmFile or $PackageManager eq "rpm" ) {
        my $request = $RpmFile ? "-qip" : "-qi";
        my $name_to_analyze = $RpmFile ? escape_shell_symbols($RpmFile) : $AppName_esc;

        # 'Distribution' field (if any) is usually the most informative
        $PkgLine = `rpm $request --nosignature $name_to_analyze | LC_ALL=C grep -F Distribution | cut -d\\: -f2- `;

        # 'Packager' and 'Vendor' fields can contain email embreced with '<>' - cut it
        if( !$PkgLine or $PkgLine =~ /\(none\)/ ) {
            $PkgLine = `rpm $request --nosignature $name_to_analyze | LC_ALL=C grep -F Vendor | cut -d\\: -f3- | cut -d\\< -f1`;
        }

        if( !$PkgLine or $PkgLine =~ /\(none\)/ ) {
            $PkgLine = `rpm -q $request --nosignature $name_to_analyze | LC_ALL=C grep -F Packager | cut -d\\: -f2- | cut -d\\< -f1`;
        }

        $packager = trim($PkgLine);
        if( !$packager ) {
            warn "Can't detect application packager!" if( !$totally_quiet );
        }
    }
    elsif( $PackageManager eq "dpkg" ) {
        # Actually, two 'Maintainer' fields can exist - 'Maintainer' and 'Original-Maintainer'
        # The former will be printed first and this is what we need. Otherwise 'Original-Maintainer'
        # is our only choise.
        $PkgLine = `dpkg -s $AppName_esc | LC_ALL=C grep -F Maintainer | cut -d\\: -f2- | cut -d\\< -f1`;
        $packager = trim($PkgLine);
        if( !$packager ) {
            warn "Can't detect application packager!"  if( !$totally_quiet );
        }
    }
    elsif( $PackageManager eq "emerge" ) {
        $packager = 'Gentoo';
    }
    elsif( $PackageManager eq "pacman" ) {
        $packager = 'ArchLinux';
    }
    else {
        warn "Can't detect application packager - package manager '$PackageManager' is not supproted" if( !$totally_quiet );
    }

    return $packager;
}

# Query package manager for app functional category.
# Return: category value
sub get_app_category()
{
    my ($line, $category);

    if( $RpmFile ) {
        my $filename = escape_shell_symbols($RpmFile);
        $line = `rpm -qivp --nosignature $filename | LC_ALL=C grep -F Group`;
        if( $line =~ /Group\s*\:\s+(\S+.*\S+)\s+Source RPM\:/ ) {
            return guess_category($1);
        }
        else {
            warn "Can't detect application functional category!" if( !$totally_quiet );
            return "";
        }
    }

    if( $DebFile ) {
        my $filename = escape_shell_symbols($DebFile);
        $line = `dpkg-deb -I $filename | LC_ALL=C grep -m1 Section `;
        if( $line =~ /Section\s*\:\s*(.+)/ ) {
            return guess_category($1);
        }
        else {
            warn "Can't detect application functional category!" if( !$totally_quiet );
            return "";
        }
    }

    my $AppName_esc = escape_shell_symbols($AppName);
    if( $PackageManager eq "rpm" ) {
        $line = `rpm -q --info $AppName_esc | LC_ALL=C grep -F Group`;
        if( $line =~ /Group\s*\:\s+(\S+.*\S+)\s+Source RPM\:/ ) {
            return guess_category($1);
        }
        else {
            warn "Can't detect application functional category!" if( !$totally_quiet );
            return "";
        }
    }

    if( $PackageManager eq "dpkg" ) {
        $line = `dpkg -s $AppName_esc | LC_ALL=C grep -m1 Section `;
        if( $line =~ /Section\s*\:\s*(.+)/ ) {
            return guess_category($1);
        }
        else {
            warn "Can't detect application functional category!" if( !$totally_quiet );
            return "";
        }
    }

    warn "Can't detect application functional category - package manager '$PackageManager' is not supproted" if( !$totally_quiet );
    return "";
}

# Translate app category ('group') obtained from package manager
# to appropriate one from our category values
sub guess_category($ )
{
    my $Category = $_[0];

    if( $Category =~ /Multimedia/i or $Category =~ /Graphics/i ) {
        return "Multimedia and Graphics";
    }
    if( $Category =~ /Games/i ) {
        return "Games";
    }
    if( $Category =~ /science/i or $Category =~ /education/i ) {
        return "Science and Education";
    }
    if( $Category =~ /Development/i ) {
        return "Development";
    }
    if( $Category =~ /Internet/i ) {
        return "Network";
    }
    if( $Category =~ /Productivity/i or $Category =~ /Editors/i or $Category =~ /Text/i ) {
        return "Office and Desktop";
    }
    if( $Category =~ /Archiving/i ) {
        return "System Tools";
    }

    return "";
}

# Given the license name, decide if this is an 'Open Source' license
sub guess_license_type($ )
{
    my $License = $_[0];

    if( $License =~ /GNU/ or $License =~ /GPL/ or $License =~ /BSD/ or $License =~ /MIT/ or $License =~ /MPL/
            or $License =~ /Public Domain/ or $License =~ /Apache Software License/ ) {
        return "Open Source";
    }
    elsif( $License =~ /Commercial/ or $License =~ /Proprietary/ ) {
        return "Proprietary";
    }

    return "";
}

# Get app name from RPM file
sub guess_rpm_appname()
{
    my $NameLine;
    my $Name;
    my $filename = escape_shell_symbols($RpmFile);

    $NameLine = `rpm -qivp --nosignature $filename | LC_ALL=C grep -F Name`;
    $Name = trim($NameLine);

    if( $Name =~ /Name\s*\:\s*(\S+)\s*Relocations\s*\:/ ) {
        return $1;
    }
    else {
        warn "Failed to obtain application name from RPM file" if(!$totally_quiet);
        return "";
    }
}

# Get app name from Deb file
sub guess_deb_appname()
{
    my $NameLine;
    my $Name;
    my $filename = escape_shell_symbols($DebFile);

    $NameLine = `dpkg-deb -I $filename | LC_ALL=C grep -F Package`;
    $Name = trim($NameLine);

    if( $Name =~ /Package\s*\:\s*(\S+)/ ) {
        return $1;
    }
    else {
        warn "Failed to obtain application name from Deb file" if(!$totally_quiet);
        return "";
    }
}

# Guess application architecture on the basis of Deb file
sub guess_deb_arch()
{
    my $ArchLine;
    my $filename = escape_shell_symbols($DebFile);
    my $arch = "";

    $ArchLine = `dpkg-deb -I $filename | LC_ALL=C grep -F Architecture`;
    $ArchLine = trim($ArchLine);

    if( $ArchLine =~ /Architecture\s*\:\s*(\S+)/ ) {
        $arch = guess_platform($1);
        if( $arch eq 'unknown' ) {
            $arch = '';
        }
    }

    return $arch;
}

# Guess application architecture on the basis of RPM file
sub guess_rpm_arch()
{
    my $arch = "";

    if( $RpmFile =~ /x86_64.rpm$/ ) {
        $arch = "x86-64";
    }
    elsif( $RpmFile =~ /i[3456]86.rpm$/ ) {
        $arch = "ia32";
    }
    elsif( $RpmFile =~ /ia64.rpm$/ ) {
        $arch = "ia64";
    }
    elsif( $RpmFile =~ /s390.rpm$/ ) {
        $arch = "s390";
    }
    elsif( $RpmFile =~ /s390x.rpm$/ ) {
        $arch = "s390x";
    }
    elsif( $RpmFile =~ /ppc.rpm$/ ) {
        $arch = "ppc32";
    }
    elsif( $RpmFile =~ /ppc64.rpm$/ ) {
        $arch = "ppc64";
    }

    return $arch;
}

# Query package manager for application license
# Return: License kind ('Open Source' or 'Proprietary')
sub get_app_license()
{
    my $LicenseLine;
    my $License;

    if( $RpmFile ) {
        my $filename = escape_shell_symbols($RpmFile);
        $LicenseLine = `rpm -qivp --nosignature $filename | LC_ALL=C grep -F License | cut -d\\: -f3- `;
        $License = trim($LicenseLine);
        if( !$License ) {
            warn "Can't detect application license type" if( !$totally_quiet );
            return "";
        }

        return guess_license_type($License);
    }

    if( $DebFile ) {
        warn "Can't detect application license type - license auto-detection is not supported for deb packages" if( !$totally_quiet );
        return "";
    }

    if( $PackageManager eq "rpm" ) {
        my $AppName_esc = escape_shell_symbols($AppName);

        $LicenseLine = `rpm -q --info $AppName_esc | LC_ALL=C grep -F License | cut -d\\: -f3- `;
        $License = trim($LicenseLine);
        if( !$License ) {
            warn "Can't detect application license type" if( !$totally_quiet );
            return "";
        }
    }
    elsif( $PackageManager eq "dpkg" or $PackageManager eq "emerge" ) {
        warn "Can't detect application license type - license autodetection is not supported for $PackageManager" if( !$totally_quiet );
        return "";
    }
    elsif( $PackageManager eq "pacman" ) {
        my $AppName_esc = escape_shell_symbols($AppName);

        $LicenseLine = `pacman -Qi $AppName_esc | LC_ALL=C grep -F License | cut -f2- -d: `;
        chomp $LicenseLine;
        if( $LicenseLine =~ /\s*(.+)\s*$/ ) {
            $License = $1;
        }
        else {
            warn "Can't detect application license type" if( !$totally_quiet );
            return "";
        }
    }
    else {
        warn "Can't detect application license - package manager '$PackageManager' is not supproted" if( !$totally_quiet );
        return "";
    }

    return guess_license_type($License);
}

# Query package manager for application version
sub get_app_version()
{
    my $VerLine;
    my $version = "";

    if( $RpmFile ) {
        my $filename = escape_shell_symbols($RpmFile);
        $VerLine = `rpm -qivp --nosignature $filename | LC_ALL=C grep -F Version | LC_ALL=C grep -F Vendor`;
        chomp($VerLine);
        if( $VerLine =~ /Version\s*\:\s+((\w|\.|\+|\-|\~|\_|\d)+)\s+Vendor/ ) {
            $version = $1;
        }
        else {
            warn "Can't detect application version!" if( !$totally_quiet );
        }

        return $version;
    }

    if( $DebFile ) {
        my $filename = escape_shell_symbols($DebFile);
        $VerLine = `dpkg-deb -I $filename | LC_ALL=C grep -F Version`;
        chomp($VerLine);
        if( $VerLine =~ /Version\s*\:\s+((\w|\.|\+|\-|\~|\_|\d)+)/ ) {
            $version = $1;
        }
        else {
            warn "Can't detect application version!" if( !$totally_quiet );
        }

        return $version;
    }

    my $AppName_esc = escape_shell_symbols($AppName);

    if( $PackageManager eq "rpm" ) {
        $VerLine = `rpm -qi $AppName_esc | LC_ALL=C grep -F Version | LC_ALL=C grep -F Vendor`;
        chomp($VerLine);
        if( $VerLine =~ /Version\s+\:\s+((\w|\.|\-|\_|\d)+)\s+Vendor/ ) {
            $version = $1;
        }
        else {
            warn "Can't detect application version!" if( !$totally_quiet );
        }
    }
    elsif( $PackageManager eq "dpkg" ) {
        $VerLine = `dpkg -s $AppName_esc | LC_ALL=C grep -i version`;
        chomp($VerLine);
        if( $VerLine =~ /(V|v)ersion\s*:\s*(\S+)/ ) {
            $version = $2;
        }
        else {
            warn "Can't detect application version!" if( !$totally_quiet );
        }
    }
    elsif( $PackageManager eq "emerge" ) {
        $VerLine = `emerge --pretend --nodeps $AppName_esc | LC_ALL=C grep $AppName_esc`;
        chomp($VerLine);
        if( $VerLine =~ /\[(\s|\w|\d)*\](\s|\w|\/|\d|\.|\-|\+)*\[((\s|\w|\d|\.|\-|\+)*)\]/ ) {
            $version = $3;
        }
        elsif( $VerLine =~ /no\ ebuilds/ ) {
            warn "Can't detect application version - it seems that it is not installed" if( !$totally_quiet );
        }
        elsif( $VerLine =~ /\Q$AppName\E(\s|\-)*((\w|\.|\d|\-|\+)+)/ ) {
            $version = $2;
        }
        else {
            warn "Can't detect application version!" if( !$totally_quiet );
        }
    }
    elsif( $PackageManager eq "pacman" ) {
        $VerLine = `pacman -Qi $AppName_esc | LC_ALL=C grep -F Version | cut -f2- -d:`;
        chomp $VerLine;
        if( $VerLine =~ /\s*(.+)\s*$/ ) {
            $version = $1;
        }
        else {
            warn "Can't detect application version" if( !$totally_quiet );
        }
    }
    else {
        warn "Can't detect application version - package manager '$PackageManager' is not supproted" if( !$totally_quiet );
    }

    return $version;
}

sub HELP_MESSAGE()
{
print STDERR <<"EOM"

$0: Obtain information about application.

Usage: $0 [options] app_name

app_name                      Name of the application. Can be absent if single rpm, deb or tarball is processed.

-a|--arch                     Specify architecture. Possible values are x86, IA64, PPC32,
                              PPC64, S390, S390X and x86-64. By default the output of `uname -m`
                              is analyzied

-h|--help                     Display this help message

-v|--version app_version      Application version. If none is specified, the script will try to obtain it from package manager

-i|--infile filename          File with the list of packages to be processed (i.e. packages that form the application)
                              If no file is specified, application name is considered to be the only package

-l|--filelist filename        File containing list of file names that are installed by the application.
                              (Directory names are also allowed; all files from given directories will be processed)

-r|--rpm filename             RPM file with application to be processed. Use this option if you don't want to
                              install the application and the whole application is provided by a single rpm file.
                              NOTE: RPM file can be given as script argument, without these options. In this case
                              application name will be taken from RPM information.

--rpmlist filename            List of RPM files containing application binaries. Use this option if you don't want to
                              install the application and the application is provided by several rpm files.

-d|--deb filename             Deb file with application to be processed. Use this option if you don't want to
                              install the application and the whole application is provided by a single deb file.
                              NOTE: deb file can be given as script argument, without these options. In this case
                              application name will be taken from deb information.

--deblist filename            List of Deb files containing application binaries. Use this option if you don't want to
                              install the application and the application is provided by several deb files.

-o|--outfile filename         Filename where the data for upload will be situated
                              By default, "<app_name>_<app_ver>_<arch>_upload_data" is used

-q|--quiet                    Be really quiet - do not produce any messages except critcal errors

--verbose                     Be verbose

--nojava                      Do not collect information about java components
                              (this data is collected by default for applications)

-u|--url URL                  Application homepage URL

-t|--licensetype type         License type. "Open Source", "Proprietary" and "Mixed" are valid values at the moment.

-p|--packager packager        Application packager - any information about uploaded application build
                              (distribution, maintainer, etc.). This can be useful since different builds of the same
                              application may require different sets of external interfaces. If nothing is specified,
                              then the script will try to guess it using package manager

--pkginfo                     Collect information about package dependencies
                              (this option has effect only if RPM packages are processed)

-m|--packagemanager pkg_name  Package manager used in the distribution. Supported values
                              are 'rpm', 'dpkg', 'emerge' and 'pacman'. If none is specified, the script
                              will try to guess it

--ui ui_kind                  User interface style. Possible values are 'non-GUI' and 'GUI'. Applications that have both
                              graphical and command line interface are treated as 'GUI'

--category category_kind      Application functional category. Possible values are:
                                  'Accessibility and i18n'
                                  'Antivirus and Security'
                                  'Data Management'
                                  'Development'
                                  'Emulators'
                                  'Games'
                                  'Multimedia and Graphics'
                                  'Network'
                                  'Office and Desktop'
                                  'Science and Education'
                                  'System Tools'
                                  'X11 Utilities'

-s|--summary summary_text     Application summary. If none is specified, the script will try to obtain it from
                              package manager; if rpm or deb file is being processed, the script will try to
                              obtain necessary data from the file itself

-c|--comment comment_text     Comment for the application. Any useful information about application build process
                              can be added here (e.g. which libraries were linked statically).

EOM
      ; exit(1);
}
